@@ -1,11 +1,14 @@
-// Copyright (c) 2011-2014 The Bitcoin developers
+// Copyright (c) 2011-2016 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #ifndef BITCOIN_QT_PAYMENTREQUESTPLUS_H
 #define BITCOIN_QT_PAYMENTREQUESTPLUS_H
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include "paymentrequest.pb.h"
+#pragma GCC diagnostic pop
 
 #include "base58.h"
 
@@ -15,6 +18,8 @@
 #include <QList>
 #include <QString>
 
+static const bool DEFAULT_SELFSIGNED_ROOTCERTS = false;
+
 //
 // Wraps dumb protocol buffer paymentRequest
 // with extra methods
